package com.deplay.one.hybrid

// import android.app.Activity
import androidx.annotation.NonNull
import com.deplay.one.hybrid.bridge.flutter.Methods
import com.deplay.one.hybrid.bridge.js.JSPageManager
import com.deplay.one.hybrid.util.TimerManager
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.*


/** FlutterPluginMethodChannel */
public class HybridPlugin : FlutterPlugin, MethodChannel.MethodCallHandler {
    override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        val channel = MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "com.deplay.one.hybrid/method")
        channel.setMethodCallHandler(HybridPlugin());
    }
    companion object {

        private lateinit var channel: MethodChannel
        fun registerWith(registrar: PluginRegistry.Registrar, methodChannelName:String) {
            channel = MethodChannel(registrar.messenger(), methodChannelName)
            channel.setMethodCallHandler(HybridPlugin())
        }

    }

    override fun onMethodCall(methodCall: MethodCall, result: MethodChannel.Result) {
        when (methodCall.method) {
            Methods.ATTACH_PAGE -> {
                if (methodCall.hasArgument("pageId") && methodCall.hasArgument("script")) {
                    val pageId = methodCall.argument<String>("pageId")
                    val script = methodCall.argument<String>("script")
                    JSPageManager.attachPageScriptToJsCore(pageId!!, script!!)
                    result.success("success")
                }
            }
            Methods.ON_LOAD -> {
                if (methodCall.hasArgument("pageId")) {
                    val pageId = methodCall.argument<String>("pageId")
                    val args = methodCall.argument<String>("args")
                    JSPageManager.callMethodInPage(pageId!!, Methods.ON_LOAD, args)
                    result.success("success")
                }
            }
            Methods.ON_UNLOAD -> {
                if (methodCall.hasArgument("pageId")) {
                    val pageId = methodCall.argument<String>("pageId")
                    TimerManager.delTimerByPageId(pageId!!)
                    JSPageManager.callMethodInPage(pageId, Methods.ON_UNLOAD)
                    JSPageManager.removePage(pageId)
                    result.success("success")
                }
            }
            Methods.EVENT -> {
                if (methodCall.hasArgument("pageId") && methodCall.hasArgument("event") && methodCall.hasArgument("data")) {
                    val pageId = methodCall.argument<String>("pageId")
                    val event = methodCall.argument<String>("event")
                    val data = methodCall.argument<String>("data")
                    JSPageManager.callMethodInPage(pageId!!, event!!, data)
                    result.success("success")
                }
            }
            Methods.ON_PULL_DOWN_REFRESH -> {
                if (methodCall.hasArgument("pageId")) {
                    val pageId = methodCall.argument<String>("pageId")
                    JSPageManager.callMethodInPage(pageId!!, Methods.ON_PULL_DOWN_REFRESH, null)
                    result.success("success")
                }
            }
            //以下是__native__内部函数回调
            Methods.INIT_COMPLETE -> {
                if (methodCall.hasArgument("pageId")) {
                    val pageId = methodCall.argument<String>("pageId")
                    JSPageManager.onInitComplete(pageId!!)
                    result.success("success")
                }
            }
            Methods.HANDLE_EXPRESSION -> {
                if (methodCall.hasArgument("pageId") && methodCall.hasArgument("id") && methodCall.hasArgument("type") && methodCall.hasArgument("key") && methodCall.hasArgument("expression")) {
                    val pageId = methodCall.argument<String>("pageId")
                    val id = methodCall.argument<String>("id")
                    val type = methodCall.argument<String>("type")
                    val key = methodCall.argument<String>("key")
                    val watch = methodCall.argument<Boolean>("watch")
                    val expression = methodCall.argument<String>("expression")
                    val obj = JSPageManager.handleExpression(pageId!!, id!!, type!!, key!!, watch!!, expression!!)
                    result.success(obj)
                }
            }
            Methods.HANDLE_REPEAT -> {
                if (methodCall.hasArgument("pageId") && methodCall.hasArgument("id") && methodCall.hasArgument("type") && methodCall.hasArgument("key") && methodCall.hasArgument("expression")) {
                    val pageId = methodCall.argument<String>("pageId")
                    val id = methodCall.argument<String>("id")
                    val type = methodCall.argument<String>("type")
                    val key = methodCall.argument<String>("key")
                    val watch = methodCall.argument<Boolean>("watch")
                    val expression = methodCall.argument<String>("expression")
                    val obj = JSPageManager.handleRepeat(pageId!!, id!!, type!!, key!!, watch!!, expression!!)
                    result.success(obj)
                }
            }
            Methods.REMOVE_OBSERVER -> {
                if (methodCall.hasArgument("pageId") && methodCall.hasArgument("ids")) {
                    val pageId = methodCall.argument<String>("pageId")
                    val ids = methodCall.argument<List<String>>("ids")
                    try {
                        JSPageManager.removeObserver(pageId!!, ids!!)
                    } catch (e: Exception) {
                        Logger.printError(e)
                    }
                    result.success("success")
                }
            }
        }
    }
    override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    }
}

