package com.deplay.one.hybrid.canvas.draw

import android.content.Context
import android.graphics.*
import java.io.ByteArrayOutputStream
import java.io.InputStream
import java.net.HttpURLConnection
import java.net.URL


class DrawImage(bitMap: Bitmap,x:Number,y:Number,width:Number,height:Number, paint: Paint?): BaseDraw()  {
    val mBitMap:Bitmap
    val dx: Float
    val dy: Float
    val mWidth:Float
    val mHeight:Float
    override fun draw(canvas: Canvas?) {
        // 获取原图尺寸
        val preWidth=mBitMap.width.toFloat()
        val preHeight=mBitMap.height.toFloat()

        // 获取接口宽高并计算缩放比例
        val sx= mWidth/preWidth;
        val sy= mHeight/preHeight;

        // 创建Matrix
        val image_Matrix = Matrix()
        // 设置尺寸
        image_Matrix.setScale(sx, sy)
        // 设置接口位移
        image_Matrix.postTranslate(dx, dy)

        //绘制图像
        canvas!!.drawBitmap(mBitMap, image_Matrix, mPaint)
        super.draw(canvas)
    }

    init {
        mPaint = paint
        mBitMap= bitMap
        dx=x.toFloat()
        dy=y.toFloat()
        mWidth=width.toFloat()
        mHeight=height.toFloat()
    }

    @Throws(Exception::class)
    fun getImage(path: String?): Bitmap? {
        val url = URL(path)
        val conn = url.openConnection() as HttpURLConnection
        conn.readTimeout = 10 * 1000
        conn.connectTimeout = 10 * 1000
        conn.requestMethod = "GET"
        var `is`: InputStream? = null
        `is` = if (conn.responseCode == HttpURLConnection.HTTP_OK) {
            conn.inputStream
        } else {
            null
        }
        return if (`is` == null) {
            throw RuntimeException("stream is null")
        } else {
            try {
                val data = readStream(`is`)
                if (data != null) {
                    return BitmapFactory.decodeByteArray(data, 0, data.size)
                }
            } catch (e: Exception) {
                e.printStackTrace()
            }
            `is`.close()
            null
        }
    }

    /*
     * 得到图片字节流 数组大小
     * */
    @Throws(Exception::class)
    fun readStream(inStream: InputStream): ByteArray? {
        val outStream = ByteArrayOutputStream()
        val buffer = ByteArray(1024)
        var len = 0
        while (inStream.read(buffer).also { len = it } != -1) {
            outStream.write(buffer, 0, len)
        }
        outStream.close()
        inStream.close()
        return outStream.toByteArray()
    }

    companion object{
        fun getImageFromAssetsFile(context: Context,fileName:String):Bitmap{
            // context.resources.assets.open(fileName);
            return BitmapFactory.decodeStream(context.resources.assets.open(fileName));
        }
    }
}