package com.deplay.one.hybrid.canvas.wrapper

import android.content.Context
import android.content.res.AssetManager
import android.graphics.*
import android.os.Environment
import android.util.Log
import com.deplay.one.hybrid.canvas.common.CanvasV8Object
import com.deplay.one.hybrid.canvas.common.CloneablePaint
import com.deplay.one.hybrid.canvas.draw.*
import com.eclipsesource.v8.*
import java.security.AccessController.getContext
import java.util.*


class CanvasContext(v8Runtime: V8, context: Context) : CanvasV8Object(v8Runtime) {
    private var mContext= context
    private var bNewStart = true
    private var mStartX = 0f
    private var mStartY = 0f
    private var mPath = Path()
    private var mPaintFill = CloneablePaint()
    private var mPaintFillSave = CloneablePaint()
    private var mPaintStroke = CloneablePaint()
    private var mPaintStrokeSave = CloneablePaint()
    val shapeList = ArrayList<AbstractDraw>()

    override protected fun initV8Object() {
        mObject = V8Object(mRuntime)
        /**
         * 属性
         */
        mObject.registerJavaMethod(this, "setFillColor", "setFillColor", arrayOf<Class<*>>(String::class.java))
        mObject.registerJavaMethod(this, "setStrokeColor", "setStrokeColor", arrayOf<Class<*>>(String::class.java))
        /**
         * 线条样式
         */
        mObject.registerJavaMethod(this, "setLineWidth", "setLineWidth", arrayOf<Class<*>>(Number::class.java))
        mObject.registerJavaMethod(this, "setLineDash", "setLineDash", arrayOf<Class<*>>(V8Array::class.java))
        /**
         * 矩形
         */
        mObject.registerJavaMethod(this, "fillRect", "fillRect", arrayOf<Class<*>>(Number::class.java, Number::class.java, Number::class.java, Number::class.java))
        mObject.registerJavaMethod(this, "strokeRect", "strokeRect", arrayOf<Class<*>>(Number::class.java, Number::class.java, Number::class.java, Number::class.java))
        /**
         * 路径
         */
        mObject.registerJavaMethod(this, "beginPath", "beginPath", null)
        mObject.registerJavaMethod(this, "closePath", "closePath", null)
        mObject.registerJavaMethod(this, "fill", "fill", null)
        mObject.registerJavaMethod(this, "stroke", "stroke", null)
        mObject.registerJavaMethod(this, "moveTo", "moveTo", arrayOf<Class<*>>(Number::class.java, Number::class.java))
        mObject.registerJavaMethod(this, "lineTo", "lineTo", arrayOf<Class<*>>(Number::class.java, Number::class.java))
        mObject.registerJavaMethod(this, "quadraticCurveTo", "quadraticCurveTo", arrayOf<Class<*>>(Number::class.java, Number::class.java, Number::class.java, Number::class.java))
        mObject.registerJavaMethod(this, "bezierCurveTo", "bezierCurveTo", arrayOf<Class<*>>(Number::class.java, Number::class.java, Number::class.java, Number::class.java, Number::class.java, Number::class.java))
        mObject.registerJavaMethod(this, "arc", "arc", arrayOf<Class<*>>(Number::class.java, Number::class.java, Number::class.java, Number::class.java, Number::class.java, Boolean::class.java))
        mObject.registerJavaMethod(this, "rect", "rect", arrayOf<Class<*>>(Number::class.java, Number::class.java, Number::class.java, Number::class.java))
        mObject.registerJavaMethod(this, "clearRect", "clearRect", arrayOf<Class<*>>(Number::class.java, Number::class.java, Number::class.java, Number::class.java))
        /**
         * 转换
         */
        mObject.registerJavaMethod(this, "scale", "scale", arrayOf<Class<*>>(Number::class.java, Number::class.java))
        mObject.registerJavaMethod(this, "rotate", "rotate", arrayOf<Class<*>>(Number::class.java))
        mObject.registerJavaMethod(this, "translate", "translate", arrayOf<Class<*>>(Number::class.java, Number::class.java))
        mObject.registerJavaMethod(this, "transform", "transform", arrayOf<Class<*>>(Number::class.java, Number::class.java, Number::class.java, Number::class.java, Number::class.java, Number::class.java))
        /**
         * 文本
         */
        mObject.registerJavaMethod(this, "setFont", "setFont", arrayOf<Class<*>>(V8Object::class.java))
        mObject.registerJavaMethod(this, "fillText", "fillText", arrayOf<Class<*>>(String::class.java, Number::class.java, Number::class.java))
        mObject.registerJavaMethod(this, "measureText", "measureText", arrayOf<Class<*>>(String::class.java))
        /**
         * 合成
         */
        mObject.registerJavaMethod(this, "setGlobalAlpha", "setGlobalAlpha", arrayOf<Class<*>>(Number::class.java))
        /**
         * 图片
         */

        mObject.registerJavaMethod(this,"drawImage","drawImage",arrayOf<Class<*>>(V8Object::class.java, Number::class.java, Number::class.java,Number::class.java,Number::class.java))
        /**
         * 其他
         */
        mObject.registerJavaMethod(this, "save", "save", null)
        mObject.registerJavaMethod(this, "restore", "restore", null)


    }

    override fun clean() {
        shapeList.clear()
        mObject.release()
    }

    fun setFillColor(color: String?) {
        mPaintFill.setRgbColor(color!!)
    }

    fun setStrokeColor(color: String?) {
        mPaintStroke.setRgbColor(color!!)
    }

    fun setLineWidth(lineWidth: Number) {
        mPaintStroke.strokeWidth = lineWidth.toFloat()
    }

    fun setLineDash(intervals: V8Array) {
        val floatIntervals = FloatArray(intervals.length())
        for (i in 0 until intervals.length()) {
            floatIntervals[i] = intervals.getDouble(i).toFloat()
        }
        val effects: PathEffect = DashPathEffect(floatIntervals, 1.toFloat())
        mPaintStroke.pathEffect = effects
        intervals.release()
    }

    fun fillRect(x: Number?, y: Number?, width: Number?, height: Number?) {
        shapeList.add(DrawRect(x!!, y!!, width!!, height!!,
                CloneablePaint(mPaintFill)))
    }

    fun strokeRect(x: Number?, y: Number?, width: Number?, height: Number?) {
        shapeList.add(DrawRect(x!!, y!!, width!!, height!!,
                CloneablePaint(mPaintStroke)))
    }

    fun beginPath() {
        mPath = Path()
        bNewStart = true
    }

    fun closePath() {
        lineTo(mStartX, mStartY)
    }

    fun fill() {
        shapeList.add(DrawPath(mPath, CloneablePaint(mPaintFill)))
    }

    fun stroke() {
        shapeList.add(DrawPath(mPath, CloneablePaint(mPaintStroke)))
    }

    fun moveTo(x: Number, y: Number) {
        bNewStart = false
        mStartX = x.toFloat()
        mStartY = y.toFloat()
        mPath.moveTo(mStartX, mStartY)
    }

    fun lineTo(x: Number, y: Number) {
        if (bNewStart) {
            moveTo(x, y)
        } else {
            mPath.lineTo(x.toFloat(), y.toFloat())
        }
    }

    fun quadraticCurveTo(cpx: Number, cpy: Number, x: Number, y: Number) {
        mPath.quadTo(cpx.toFloat(), cpy.toFloat(),
                x.toFloat(), y.toFloat())
    }

    fun bezierCurveTo(cp1x: Number, cp1y: Number, cp2x: Number, cp2y: Number, x: Number, y: Number) {
        mPath.cubicTo(cp1x.toFloat(), cp1y.toFloat(),
                cp2x.toFloat(), cp2y.toFloat(),
                x.toFloat(), y.toFloat())
    }

    fun arc(x: Number, y: Number, r: Number, sAngle: Number, eAngle: Number, counterclockwise: Boolean) {
        var sAngle = sAngle
        var eAngle = eAngle
        val rectF = RectF(x.toFloat() - r.toFloat(), y.toFloat() - r.toFloat(),
                x.toFloat() + r.toFloat(), y.toFloat() + r.toFloat())
        //弧度与角度转换
        sAngle = (sAngle.toFloat() / Math.PI * 180)
        eAngle = (eAngle.toFloat() / Math.PI * 180)
        if (counterclockwise) {
            sAngle = -sAngle.toFloat()
            eAngle = -eAngle.toFloat()
        }
        mPath.addArc(rectF, sAngle.toFloat(), eAngle.toFloat() - sAngle.toFloat())
    }

    fun rect(x: Number, y: Number, width: Number, height: Number) {
        mPath.addRect(x.toFloat(), y.toFloat(),
                x.toFloat() + width.toFloat(), y.toFloat() + height.toFloat(),
                Path.Direction.CW)
    }

    fun clearRect(x: Number?, y: Number?, width: Number?, height: Number?) {}

    fun drawImage(image:V8Object,x: Number, y: Number, width: Number, height: Number){

        val src:String= image.getString("src");

        val Drawed = DrawImage(DrawImage.getImageFromAssetsFile(mContext,src),x,y,width,height,CloneablePaint(mPaintFill))


        image.add("width",Drawed.mWidth.toInt())
        image.add("height",Drawed.mHeight.toInt())
        image.registerJavaMethod(JavaCallback { receiver, parameters ->
            val jsCallback = parameters?.getObject(0)
            if(jsCallback is V8Function){
                jsCallback.call(receiver, V8Array(image.getRuntime()).push("successful"));
            }
            if (jsCallback is Releasable) {
                jsCallback.release()
            }
            receiver as Any
        }, "onLoad")
        Log.i("mStatus",Drawed.getStatus().toString())
        shapeList.add(Drawed)
    }

    fun scale(scaleWidth: Number?, scaleHeight: Number?) {
        shapeList.add(DrawScale(scaleWidth!!, scaleHeight!!))
    }

    fun rotate(angle: Number?) {
        shapeList.add(DrawRotate(angle!!))
    }

    fun translate(x: Number?, y: Number?) {
        shapeList.add(DrawTranslate(x!!, y!!))
    }

    fun transform(a: Number?, b: Number?, c: Number?, d: Number?, dx: Number?, dy: Number?) {
        shapeList.add(DrawTransform(a!!, b!!, c!!, d!!, dx!!, dy!!))
    }

    fun setFont(font: V8Object) {
        mPaintFill.textSize = font.getInteger("size").toFloat()
        font.release()
    }

    fun fillText(text: String?, x: Number?, y: Number?) {
        shapeList.add(DrawText(text!!, x!!, y!!, CloneablePaint(mPaintFill)))
    }

    fun measureText(str: String): V8Object {
        val rect = Rect()
        mPaintFill.getTextBounds(str, 0, str.length, rect)
        val result = V8Object(mRuntime)
        result.add("width", rect.width())
        result.add("height", rect.height())
        return result
    }

    fun setGlobalAlpha(alpha: Number) {
        mPaintFill.alpha = (alpha.toInt() * 255)
        mPaintStroke.alpha = (alpha.toInt() * 255)
    }

    fun save() {
        shapeList.add(DrawSave())
        mPaintFillSave = CloneablePaint(mPaintFill)
        mPaintStrokeSave = CloneablePaint(mPaintStroke)
    }

    fun restore() {
        shapeList.add(DrawRestore())
        mPaintFill = CloneablePaint(mPaintFill)
        mPaintStroke = CloneablePaint(mPaintStroke)
    }

    init {

        mPaintFill.style = Paint.Style.FILL
        mPaintStroke.style = Paint.Style.STROKE
        mPaintFill.flags = Paint.ANTI_ALIAS_FLAG
        mPaintStroke.flags = Paint.ANTI_ALIAS_FLAG
    }
}