package com.deplay.one.hybrid.canvas.wrapper

import android.content.Context;
import com.deplay.one.hybrid.canvas.common.CanvasConfig
import com.deplay.one.hybrid.canvas.common.CanvasV8Object
import com.deplay.one.hybrid.canvas.view.CanvasView
import com.deplay.one.hybrid.util.TimerManager
import com.eclipsesource.v8.JavaCallback

import com.eclipsesource.v8.V8;
import com.eclipsesource.v8.V8Object;

class CanvasMain(v8Runtime: V8, context: Context) : CanvasV8Object(v8Runtime) {
    var mCanvasView: CanvasView
    var mCanvasContext: CanvasContext
    override  protected fun initV8Object() {
        mObject = V8Object(mRuntime)
        mObject.registerJavaMethod(this, "setX", "setX", arrayOf<Class<*>>(Number::class.java))
        mObject.registerJavaMethod(this, "setY", "setY", arrayOf<Class<*>>(Number::class.java))
        mObject.registerJavaMethod(this, "setWidth", "setWidth", arrayOf<Class<*>>(Number::class.java))
        mObject.registerJavaMethod(this, "setHeight", "setHeight", arrayOf<Class<*>>(Number::class.java))
        mObject.registerJavaMethod(this, "setSize", "setSize", arrayOf<Class<*>>(Number::class.java, Number::class.java, Number::class.java, Number::class.java))

//        exports.requestAnimationFrame = requestAnimationFrame;
//        exports.cancelAnimationFrame = cancelAnimationFrame;

        mObject.add("x", 0)
        mObject.add("y", 0)
        mObject.add("width", CanvasConfig.screenWidth)
        mObject.add("height", CanvasConfig.screenHeight - CanvasConfig.navbarheight - CanvasConfig.statusbarheight)
    }

    override fun clean() {
        mCanvasContext.clean()
        mObject.release()
    }

    val canvasView: CanvasView
        get() = mCanvasView

    fun drawCanvas() {
        mCanvasView.draw(
                mCanvasContext.shapeList)
    }

    fun invalidate() {
        mCanvasView.invalidate()
    }

    fun setX(x: Number) {
        mCanvasView.canvasX = x.toFloat()
    }

    fun setY(y: Number) {
        mCanvasView.canvasY = y.toFloat()
    }

    fun setWidth(width: Number) {
        mCanvasView.canvasWidth = width.toFloat()
    }

    fun setHeight(height: Number) {
        mCanvasView.canvasHeight = height.toFloat()
    }

    fun setSize(x: Number, y: Number, width: Number, height: Number) {
        setX(x)
        setY(y)
        setWidth(width)
        setHeight(height)
    }

    init {
        mCanvasView = CanvasView(context)
        mCanvasContext = CanvasContext(v8Runtime,context)
        mObject.add("context", mCanvasContext.getObject())
    }
}