package com.deplay.one.hybrid.canvas.wrapper

import android.graphics.Paint
import android.graphics.Rect
import android.util.Log
import com.deplay.one.hybrid.Logger
import com.deplay.one.hybrid.canvas.common.CanvasConfig
import com.deplay.one.hybrid.canvas.common.CanvasV8Object
import com.deplay.one.hybrid.canvas.view.CanvasViewGroup
import com.deplay.one.hybrid.util.SpUtil
import com.deplay.one.hybrid.util.TimerManager
import com.eclipsesource.v8.*
import java.util.*

class CanvasWindow(v8Runtime: V8, viewGroup: CanvasViewGroup) : CanvasV8Object(v8Runtime) {
    lateinit var mV8Object:V8Object

    fun getV8Object(): V8Object? {
        return mV8Object
    }
    var mCanvasViewGroup: CanvasViewGroup = viewGroup
    var mCanvasMainList: ArrayList<CanvasMain> = ArrayList<CanvasMain>()
    override fun initV8Object() {
        mV8Object = V8Object(mRuntime)
        mV8Object.add("devicePixelRatio", CanvasConfig.density.toDouble())
        mRuntime.registerJavaMethod(JavaCallback { receiver, parameters ->
            val pageId = parameters.getString(0)
            val timerId = parameters.getString(1)
            val delayed = parameters?.getInteger(2)
            if (null != delayed) {
                TimerManager.setTimeout(pageId, timerId, delayed)
            }
            receiver as Any
        }, "setTimeout")
        mRuntime.registerJavaMethod(JavaCallback { receiver, parameters ->
            val timerId = parameters.getString(0)
            TimerManager.delTimer(timerId)
            receiver as Any
        }, "clearTimeout")
        mRuntime.registerJavaMethod(JavaCallback { receiver, parameters ->
            val pageId = parameters.getString(0)
            val timerId = parameters.getString(1)
            val delayed = parameters?.getInteger(2)
            if (null != delayed) {

                // TimerManager.setInterval(pageId, timerId, delayed)
            }
            receiver as Any
        }, "setInterval")
        mRuntime.registerJavaMethod(JavaCallback { receiver, parameters ->
            val timerId = parameters.getString(0)
            TimerManager.delTimer(timerId)
            receiver as Any
        }, "clearInterval")



        mV8Object.registerJavaMethod(this, "onTouchStart", "onTouchStart", null)
        mV8Object.registerJavaMethod(this, "onTouchMove", "onTouchMove", null)
        mV8Object.registerJavaMethod(this, "onTouchEnd", "onTouchEnd", null)
        mV8Object.registerJavaMethod(this, "onTouchCancel", "onTouchCancel", null)

        mV8Object.registerJavaMethod(this, "createCanvas", "createCanvas", null)
        mV8Object.registerJavaMethod(this, "measureText", "measureText", arrayOf(String::class.java, V8Object::class.java))
        mV8Object.registerJavaMethod(this, "getSystemInfoSync", "getSystemInfoSync", null)
        mV8Object.registerJavaMethod(this, "createImage","createImage",null);
    }

    override  fun clean() {
        for (canvasMain in mCanvasMainList) {
            canvasMain.clean()
        }
        mV8Object.release()
    }

    fun drawWindow() {
        for (canvasMain in mCanvasMainList) {
            canvasMain.drawCanvas()
        }
    }

    fun invalidate() {
        for (canvasMain in mCanvasMainList) {
            mCanvasViewGroup.addView(canvasMain.canvasView)
            canvasMain.invalidate()
        }
    }

    fun createCanvas(): V8Object {
        val canvasMain = CanvasMain(mV8Object.getRuntime(), mCanvasViewGroup.getContext())
        mCanvasMainList.add(canvasMain)
        return canvasMain.getObject();
    }

    fun measureText(str: String, font: V8Object): V8Object { //TODO
        val paint = Paint()
        paint.textSize = font.getInteger("size").toFloat()
        val rect = Rect()
        paint.getTextBounds(str, 0, str.length, rect)
        val result = V8Object(mV8Object.getRuntime())
        result.add("width", rect.width())
        result.add("height", rect.height())
        return result
    }

    fun getSystemInfoSync(): V8Object{
        val result = V8Object(mV8Object.getRuntime())
        result.add("platform","android")
        return result;
    }

    fun onTouchStart(){
    }
    fun onTouchMove(){
    }
    fun onTouchEnd(){
    }
    fun onTouchCancel(){
    }

    fun createImage():V8Object{
        val result = V8Object(mV8Object.getRuntime())
        result.add("src","")
        return result
    }

    init {
        mCanvasViewGroup = viewGroup
    }



}