//
//  JSCoreNetwork.m
//  Runner
//
//  Created by 苏玮 on 2019/12/13.
//  Copyright © 2019 The Chromium Authors. All rights reserved.
//

#import "JSCoreNetwork.h"
#import <AFNetworking/AFNetworking.h>

typedef NS_ENUM(NSInteger,RequestBase){
    REQUEST_HEADER,
    REQUEST_BODY,
    REQUEST_URL,
    REQUEST_METHOD,
    REQUEST_PROTOCOL,
    REQUEST_DATA,
};

typedef NS_ENUM(NSInteger,ResponseStatus){
    RESPONSE_HANDSHAKE,
    RESPONSE_SUCCESS,
    RESPONSE_FAIL,
    RESPONSE_RESULT,
    RESPONSE_COMPLETE,
};

typedef NS_ENUM(NSInteger,Payload){
    PAYLOAD_REQUESTID,
    PAYLOAD_PAGEID,
    PAYLOAD_MESSAGE,
    PAYLOAD_HEADERS,
};

@implementation JSCoreNetwork

NSString *getRequestBaseEnum(RequestBase base) {
    switch (base) {
        case REQUEST_HEADER:
            return @"header";
        case REQUEST_BODY:
            return @"body";
        case REQUEST_URL:
            return @"url";
        case REQUEST_METHOD:
            return @"method";
        case REQUEST_PROTOCOL:
            return @"protocol";
        case REQUEST_DATA:
            return @"data";
        default:
            return @"";
    }
}


NSString *getRequestStatusEnum(ResponseStatus status) {
    switch (status) {
        case RESPONSE_HANDSHAKE:
            return @"Handshake";
        case RESPONSE_SUCCESS:
            return @"success";
        case RESPONSE_FAIL:
            return @"fail";
        case RESPONSE_COMPLETE:
            return @"complete";
        default:
            return @"";
    }
}

NSString *getPayloadEnum(Payload payload) {
    switch (payload) {
        case PAYLOAD_REQUESTID:
            return @"requestId";
        case PAYLOAD_PAGEID:
            return @"pageId";
        case PAYLOAD_MESSAGE:
            return @"message";
        case PAYLOAD_HEADERS:
            return @"headers";
        default:
            return @"";
    }
}


-(void) jSCoreRequest:(JSValue*) request
     WithSuccessBlock:(requestSuccessBlock)success
      WithFailurBlock:(requestFailureBlock)failure
{
    JSValue *headerRaw=request[@"header"];
    JSValue *bodyRaw= request[@"data"];
    
    
    NSDictionary *header= headerRaw.toDictionary;
    NSString *url = request[@"url"].toString;
    NSString *method= request[@"method"].toString;
    NSDictionary *body= bodyRaw.toDictionary;
    
    AFHTTPRequestSerializer *_serializer =[AFHTTPRequestSerializer serializer];
    HTTPMethod _requestMethod = GET;
   
    
    if([method isEqualToString:@"post"]){
        _requestMethod = POST;
    }
    
    if(headerRaw.isNull||headerRaw.isUndefined){
        header = @{};
    }
    
    if(bodyRaw.isNull|| bodyRaw.isUndefined){
        body = @{};
    }
    
    if([header[@"content-type"] isEqualToString:@"application/json"]){
      _serializer = [AFJSONRequestSerializer serializer];
    }
    
    AFHTTPSessionManager *manager =[[AFHTTPSessionManager alloc] init];
  
    
    
    manager.requestSerializer = _serializer;
    manager.requestSerializer.timeoutInterval = 5;
    manager.requestSerializer.cachePolicy = NSURLRequestReloadIgnoringLocalCacheData;
    

    manager.responseSerializer.acceptableContentTypes =
        [NSSet setWithObjects:
                                 @"application/json",
                                 @"text/plain",
                                 @"text/javascript",
                                 @"text/json",
                                 @"text/html",
                                 nil];
    manager.securityPolicy.allowInvalidCertificates = YES;

    
    
    switch (_requestMethod) {
        case GET:{
            [manager GET:url parameters:body progress:nil success:^(NSURLSessionTask *task, NSDictionary * responseObject) {
               NSMutableDictionary *newResult=[[NSMutableDictionary alloc]init];
                [newResult setValue:@"OK" forKey:@"message"];
                [newResult setValue:responseObject forKey:@"body"];
                [newResult setValue:[NSNumber numberWithInt:200] forKey:@"code"];
                success(newResult);
            } failure:^(NSURLSessionTask *operation, NSError *error) {
                NSLog(@"Error: %@", error);
                failure(error);
            }];
            break;
        }
        case POST:{
            [manager POST:url parameters:body progress:nil success:^(NSURLSessionTask *task, NSDictionary * responseObject) {
                success(responseObject);
            } failure:^(NSURLSessionTask *operation, NSError *error) {
                failure(error);
            }];
            break;
        }
        default:
            break;
    }
    
    
    
}

@end
