//
//  JSCorePageManager.m
//  Runner
//
//  Created by 苏玮 on 2019/12/9.
//  Copyright © 2019 The Chromium Authors. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <JavaScriptCore/JavaScriptCore.h>
#import "EventEmitter.h"
#import "JSCoreEngine.h"
#import "JSCorePageManager.h"
#import "JSCoreNetwork.h"

@implementation JSCorePageManager{
    // JSContext getter
    // NSMutableDictionary *_pageDictionary;
    JSCoreEngine *_jsCoreEngine;
    NSObject *_emitter;
}


- (instancetype)init:(JSCoreEngine*)engine
             emitter:(NSObject* _Nullable )emitter
{
    if (self = [super init])
    {
        [self setJSCoreEngine:engine];
        self.pageDictionary = [[NSMutableDictionary alloc] init];
        self.currentPageId =[[NSString alloc] init];
        self.network = [[JSCoreNetwork alloc] init];
        
        if( emitter != NULL ){
            _emitter = emitter;
        }else {
           _emitter =  [[NSObject alloc] init];
        }
       
        [self->_emitter emit:@"ready"];
    }

    return self;
}


- (JSCoreEngine *)jsCoreEngine
{
    return _jsCoreEngine;
}

- (JSContext *) globalContext
{
    return self.jsCoreEngine.jsExecutor.jsContext;
}

- (JSCoreExecutor *) globalExecutor
{
    return self.jsCoreEngine.jsExecutor;
}


-(void) setJSCoreEngine:(JSCoreEngine*)engine{
    _jsCoreEngine = engine;
}

-(NSString*)prepareScriptContent:(NSString *) content
{
    return [content stringByReplacingOccurrencesOfString:@"\n" withString:@""];
}

-(void) attachPageScriptToJsCore:(NSString *) pageId
                  scriptToAttach:(NSString *) script
{
    [self attachPage:pageId scriptToAttach:[self prepareScriptContent:script]];
}

-(void) attachPage:(NSString *) pageId
    scriptToAttach:(NSString *) script
{
    JSContext *globalContext= self.globalContext;
    
    
    // load page
    [globalContext[@"global"] invokeMethod:@"loadPage" withArguments:@[pageId]];
    
    // get realpage
    JSValue *realPageObject = [self getJSPage:pageId];
    
    if(NULL != realPageObject && !realPageObject.isNull && !realPageObject.isUndefined){
        // run page script
        [realPageObject.context evaluateScript:script];
        
        // get page
        JSValue *page = [realPageObject.context objectForKeyedSubscript:@"page"];
        
        // put page into realpage object
        page = [self setProtoType:page use:realPageObject];
        
        // set prototype of page inside realpage object
        realPageObject = [self setPages:page jsPage:realPageObject];
        
        realPageObject[@"__native__refresh"]= ^(JSValue *parameters){
            JSValue *data = parameters;
            JSValue *localPageId = page[@"pageId"];
            [self->_emitter emit:@"TYPE_REFRESH" data:@{
                @"pageId": localPageId.toString,
                @"data":data.toString,
            }];
        };
        
        self.currentPageId = page[@"pageId"].toString;

        // set method for cc
        JSValue *cc = realPageObject[@"cc"];
         // __weak JSValue *weakCC = cc;
         __strong JSValue *strongCC = cc;
       
        
        // __native__refresh
        cc[@"__native__refresh"] = ^(JSValue *parameters) {
                   // NSLog(@"showLoading receiver %@",receiver.toObject);
            // NSLog(@"showToast parameters %@",parameters.toObject);
            // JSValue *localPageId = page[@"pageId"];
            NSString * localPageId =self.currentPageId;
            // NSLog(@"local page id %@",localPageId);
            JSValue *data = [self.globalContext[@"JSON"][@"stringify"] callWithArguments:@[parameters]];
            [self onRefresh:localPageId data:data.toString];
        };
        
        // setNavigationBarTitle
        cc[@"setNavigationBarTitle"] = ^(JSValue *parameters) {
            
            // NSLog(@"setNavigationBarTitle parameters %@",parameters.toObject);
            // JSValue *localPageId = page[@"pageId"];
            NSString * localPageId =self.currentPageId;
            // NSLog(@"local page id %@",localPageId.toString);
            JSValue *data = parameters;
             if(!data.isNull && data!= NULL && !data[@"title"].isNull){
                [self->_emitter emit:@"TYPE_NAVIGATION_BAR_TITLE" data:@{
                    @"pageId": localPageId,
                    @"data":data[@"title"].toString,
                }];
            }
        };

        // setNavigationBarColor
        cc[@"setNavigationBarColor"] = ^(JSValue *parameters) {
            
            // NSLog(@"setNavigationBarColor parameters %@",parameters.toObject);
            // JSValue *localPageId = page[@"pageId"];
            NSString * localPageId =self.currentPageId;
            JSValue *data = parameters;
            if(!data.isNull && data!= NULL && !data[@"backgroundColor"].isNull){
                [self->_emitter emit:@"TYPE_NAVIGATION_BAR_COLOR" data:@{
                    @"pageId": localPageId,
                    @"data":data[@"backgroundColor"].toString,
                }];
            }
        };

        // setNavigationBarColor
        cc[@"setBackgroundColor"] = ^(JSValue *parameters) {
            // NSLog(@"setBackgroundColor receiver %@",receiver.toObject);
            // NSLog(@"setBackgroundColor parameters %@",parameters.toObject);
            // JSValue *localPageId = page[@"pageId"];
            NSString * localPageId =self.currentPageId;
            JSValue *data = parameters;
            if(!data.isNull && data!= NULL && !data[@"backgroundColor"].isNull){
                [self->_emitter emit:@"TYPE_BACKGROUND_COLOR" data:@{
                    @"pageId": localPageId,
                    @"data":data[@"backgroundColor"].toString,
                }];
            }
        };
        // setNavigationBarColor
        cc[@"showLoading"] = ^(JSValue *parameters) {
            // NSLog(@"showLoading receiver %@",receiver.toObject);
            NSLog(@"showLoading parameters %@",parameters.toObject);
            
        };
        
        
        
        
        cc[@"hideLoading"] = ^(JSValue *parameters) {
            // NSLog(@"showLoading receiver %@",receiver.toObject);
            NSLog(@"hideLoading parameters %@",parameters.toObject);
        };
        cc[@"showToast"] = ^(JSValue *parameters) {
                   // NSLog(@"showLoading receiver %@",receiver.toObject);
            NSLog(@"showToast parameters %@",parameters.toObject);
        };
        
        
        cc[@"navigateTo"] = ^(JSValue *parameters) {
            NSString *localPageId = self.currentPageId;
//            NSLog(@"wtf pageId  %@",pageId);
//            NSLog(@"localPageId  %@",localPageId);
//            NSLog(@"realpage cc  %@",realPageObject[@"pageId"].toString);
//            NSLog(@"weak cc %@",weakCC[@"pageId"].toString);
//            NSLog(@"strong cc  %@",strongCC[@"pageId"].toString);
//            NSLog(@"self currentPageId  %@",self.currentPageId);
           
            
            
            
            JSValue *data = parameters;
            if(!data.isNull && data!= NULL){
                [self.globalContext evaluateScript:@"function toAssign(source){return JSON.stringify(Object.assign({},source));};"];
                JSValue *assigned=[self.globalContext[@"toAssign"] callWithArguments:@[data]];
                // NSLog(@"assigned %@",assigned.toString);
                [self->_emitter emit:@"TYPE_NAVIGATE_TO" data:@{
                    @"pageId": localPageId,
                    @"data":assigned.toString,
                }];
            }
        };
        
        cc[@"request"] = ^(JSValue *parameters) {
            parameters[@"requestId"]=[self getUUID];
            parameters[@"pageId"]=page[@"pageId"];
            NSString *key=parameters[@"requestId"].toString;
            [globalContext evaluateScript:@"var kvSetter=function(key,value){var obj=new Object();obj[key]=value;return obj;}"];
            JSValue *requestData=[globalContext[@"kvSetter"] callWithArguments:@[key,parameters]];
            strongCC[@"requestData"]=requestData;
            
            
            [self.network jSCoreRequest:parameters WithSuccessBlock:^(NSDictionary * _Nonnull dic) {
                // NSLog(@"Successful %@",dic);
                
                NSString *json = [self dictToJson:dic];
                JSValue *requestId = parameters[@"requestId"];
                
               //  NSLog(@"requestId %@", requestId.toString);
                NSString *success = @"success";
                
                // NSLog(@"page is  %@", page.toObject);
                
                JSValue *thePage=[self getJSPage:page[@"pageId"].toString];
                // NSLog(@"request param %@",parameters.toDictionary);
                
                [thePage[@"cc"] invokeMethod:@"onNetworkResult" withArguments:@[requestId,success,json]];
                
            } WithFailurBlock:^(NSError * _Nonnull error) {
                NSLog(@"failed %@",error);
            }];
            
            
        };
        globalContext[@"global"][@"cc"] = cc;
        
    }
}

-(JSValue *) getJSPage:(NSString *) pageId
{
    JSValue *cache =[self.pageDictionary valueForKey:pageId];
    if(cache != NULL && !cache.isUndefined && !cache.isNull){
        return cache;
    }else{
        [self.pageDictionary removeObjectForKey:pageId];
    }
    // NSLog(@"prepare To get page use js");
    JSValue *page = [self getPage:pageId];
    // NSLog(@"fuck page%@",page.toObject);
    if(page!= NULL){
        [self.pageDictionary setValue:page forKey:pageId];
        return page;
    }else{
        return NULL;
    }
}

- (JSValue *) getPage:(NSString *) pageId
{
   //  NSLog(@"global context%@",self.globalContext[@"getPage"].toDictionary);
    NSString *pageJS = @"this.getPage('pageId')";
    NSString *range = @"pageId";
    NSRange rang = [pageJS rangeOfString:range];
    NSString *str = [pageJS stringByReplacingCharactersInRange:rang withString:pageId];
    return [self.jsCoreEngine.jsExecutor executeJS:str];
}


- (JSValue *) setProtoType:(JSValue *) from
                      use:(JSValue *) use
{
    NSString *jsScript=@"var setProtoType=function(a,b){var result=Object.setPrototypeOf(a,b);return result;}";
    [self.globalExecutor executeJS:jsScript];
    JSValue *result=[self.globalContext[@"setProtoType"] callWithArguments:@[from,use]];
    return result;
}

- (JSValue *) setPages:(JSValue *) pageValue
               jsPage:(JSValue *) jsPage
{
    
    NSString *jsScript=@"var setPages=function(a,b){var keys=Object.keys(a);keys.forEach(function(key){b[key] = a[key];Object.setPrototypeOf(b, a[key]);});return b;}";
    [self.globalExecutor executeJS:jsScript];
    JSValue *result=[self.globalContext[@"setPages"] callWithArguments:@[pageValue,jsPage]];
    return result;
}


-(void) callMethodInPage:(NSString *) pageId
                  method:(NSString *) method
                  varArg:(NSString * _Nullable) args
{
    @try {
        JSValue *page= [self getJSPage:pageId];
        
        if(![method isEqualToString:@""]&& !page.isNull && page != NULL && !page.isUndefined && !page[method].isNull)
        {
            
            JSValue *params= [self.globalContext evaluateScript:@"new Array();"];

            if(args!=NULL && ![args isEqualToString:@""]){
                JSValue *param=[self.globalContext.globalObject[@"JSON"][@"parse"] callWithArguments:@[args]];
                if (@available(iOS 9.0, *)) {
                    if(param.isArray || param.isObject || param.isString || param.isNumber || param.isBoolean){
                        [params invokeMethod:@"push" withArguments:@[param]];
                    }
                } else {
                    if(param.isObject || param.isString || param.isNumber || param.isBoolean){
                        [params invokeMethod:@"push" withArguments:@[param]];
                    }
                    // Fallback on earlier versions
                }
            }
            self.currentPageId = page[@"pageId"].toString;
            [page invokeMethod:method withArguments:params.toArray];

        }
        
    } @catch (NSException *exception) {
        NSLog(@"callMethodInPage error %@", exception);
    }
}


 -(NSString *)dictToJson:(NSDictionary*)dic
 {
     NSError *error = nil;
     NSData *jsonData = [NSJSONSerialization dataWithJSONObject:dic
                                                        options:NSJSONWritingPrettyPrinted
                                                          error:&error];
     if (error)
     {
        NSLog(@"error");
     }
     NSString *jsonReturned = [[NSString alloc]initWithData:jsonData encoding:NSUTF8StringEncoding];
     return jsonReturned;
 }


-(NSString *)getUUID {
    CFUUIDRef puuid = CFUUIDCreate( nil );
    CFStringRef uuidString = CFUUIDCreateString(nil, puuid);
    NSString *result = (NSString *)CFBridgingRelease(CFStringCreateCopy( NULL, uuidString));
    return result;
}





-(JSValue *) handleRepeat:(NSString*)pageId
         componentId:(NSString*)componentId
                type:(NSString*)type
                 key:(NSString*)key
               watch:(NSNumber*)watch
          expression:(NSString*)expression
{
    JSValue *page= [self getJSPage:pageId];
    JSValue *watchVal;
    
    if([watch intValue]==1){
        watchVal=[self.globalContext evaluateScript:@"true"];
    }else if([watch intValue]==0){
        watchVal=[self.globalContext evaluateScript:@"false"];
    }
        
    JSValue *result =[page invokeMethod:@"__native__getExpValue" withArguments:@[componentId,type,key,watchVal,expression]];
    return result;
       
}

-(NSString *) handleExpression:(NSString*)pageId
         componentId:(NSString*)componentId
                type:(NSString*)type
                 key:(NSString*)key
               watch:(NSNumber*)watch
          expression:(NSString*)expression
{
    JSValue *page= [self getJSPage:pageId];
    JSValue *watchVal;
    
    if([watch intValue]==1){
        watchVal=[self.globalContext evaluateScript:@"true"];
    }else if([watch intValue]==0){
        watchVal=[self.globalContext evaluateScript:@"false"];
    }
        
    JSValue *result =[page invokeMethod:@"__native__getExpValue" withArguments:@[componentId,type,key,watchVal,expression]];
    return result.toString;
}


-(void) onInitComplete:(NSString*)pageId
{
     JSValue *page= [self getJSPage:pageId];
     [page invokeMethod:@"__native__initComplete" withArguments:@[]];
}

-(void) onRefresh:(NSString*)pageId
             data:(NSString*)data
{
     [self->_emitter emit:@"TYPE_REFRESH" data:@{
         @"pageId": pageId,
         @"data":data,
     }];
}
-(void) onRemovePage:(NSString*)pageId
{
     [self.globalContext[@"global"] invokeMethod:@"removePage" withArguments:@[pageId]];
}

-(void) onRemoveObserver:(NSString*)pageId ids:(NSArray*)ids
{
    JSValue *page= [self getJSPage:pageId];
    [page invokeMethod:@"__native__removeObserverByIds" withArguments:ids];
}

-(void) callback:(NSString*)callbackId
{
    [self.globalContext[@"callback"] callWithArguments:@[callbackId]];
}





@end
