#import "HybridPlugin.h"



@implementation HybridPlugin{
    BOOL _emitterReady;
}

- (void)viewDidLoad
{
    // [super viewDidLoad];
    // event emitter before others
    [self initEmitter];
    [self setupEventhandler];

    [self initJSEngine];
    [self setupBasicChannel];
    [self setupMethodChannel];
    [self setupMethodHandler];

}

- (instancetype)init
{
    [self initEmitter];
    [self setupEventhandler];
    [self initJSEngine];
    [self setupBasicChannel];
    [self setupMethodChannel];
    [self setupMethodHandler];
    return self;
}

- (instancetype)initWithRegister:(NSObject<FlutterPluginRegistrar>*)registrar
{
    [self initEmitter];
    [self setupEventhandler];
    [self initJSEngine];
    [self setupBasicChannel:registrar];
    [self setupMethodChannel:registrar];
    [self setupMethodHandler];
    return self;
}

+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar{
    [[HybridPlugin alloc] initWithRegister:registrar];
    
//   [self initEmitter];
//   [self setupEventhandler];
//   [self initJSEngine];
//   [self setupBasicChannel];
//   [self setupMethodChannel];
//   [self setupMethodHandler];
}

- (void) initEmitter{
    self->_emitter = [[NSObject alloc] init];
    self->_emitterReady = NO;
}

- (void) initJSEngine{
    self->_jsEngine = [[JSCoreEngine alloc] init];
    self->_pageManager = [[JSCorePageManager alloc] init:self.jsEngine emitter:self.emitter];
}


- (void) setupEventhandler
{
    [self->_emitter on:@"ready" notify:^() {
        NSLog(@"Yepp! The object is ready");
        self->_emitterReady = YES;
    }];
    
    [self->_emitter on:@"TYPE_REFRESH" callback:^(NSDictionary* data) {
        if(self->_emitterReady){
            NSLog(@"event:refresh");
            NSDictionary  *dict = @{
                @"type":@1,
                @"pageId": data[@"pageId"],
                @"message":data[@"data"],
            };
            [self.basicChannel sendMessage:[self.pageManager dictToJson:dict]];
        }
       
         }];
    [self.emitter on:@"TYPE_NAVIGATION_BAR_TITLE" callback:^(NSDictionary* data) {
           if(self->_emitterReady){
               // NSLog(@"Receive event with data: %@", data);
               NSLog(@"event:setNavigationBarTitle");
               NSDictionary  *dict = @{
                   @"type":@2,
                   @"pageId": data[@"pageId"],
                   @"message":data[@"data"],
               };
               [self.basicChannel sendMessage:[self.pageManager dictToJson:dict]];
           }
         }];
    [self.emitter on:@"TYPE_NAVIGATE_TO" callback:^(NSDictionary* data) {
        if(self->_emitterReady){
             NSLog(@"event:nagivateTo");
            // NSLog(@"Receive event with data: %@", data);
            NSDictionary  *dict = @{
                @"type":@3,
                @"pageId": data[@"pageId"],
                @"message":data[@"data"],
            };
            [self.basicChannel sendMessage:[self.pageManager dictToJson:dict]];
        }
       
    }];
}


// 设置BasicChannel
// 注意：在Flutter和Native通信中，BasicChannel是一个单向通道，而MethodChannel是双工的，而MethodChannel是异步的
// 这里使用Basic和Method一起，确保同步和异步可以同时进行
// 同时使用Codec为String，即`FlutterStringCodec`，在Flutter侧也是用Codex为String
// TODO： 这里需要把basicChannelName变成const
- (void) setupBasicChannel{
    // FlutterViewController* controller = self;
    self.basicChannel = [FlutterBasicMessageChannel messageChannelWithName:@"com.deplay.one.hybrid/basic" binaryMessenger:[self binaryMessenger] codec:[FlutterStringCodec new]];

}

- (void) setupMethodChannel{
    // FlutterViewController* controller = self;
   self.methodChannel = [FlutterMethodChannel methodChannelWithName:@"com.deplay.one.hybrid/method"
    binaryMessenger:[self binaryMessenger]];

}


// 设置MethodChannel
// TODO： 这里需要把methodChannelName变成const
- (void) setupMethodChannel:(NSObject<FlutterPluginRegistrar>*)registrar{
    // FlutterViewController* controller = self;
   self.methodChannel = [FlutterMethodChannel methodChannelWithName:@"com.deplay.one.hybrid/method"
    binaryMessenger:[registrar messenger]];

}

- (void) setupBasicChannel:(NSObject<FlutterPluginRegistrar>*)registrar{
    // FlutterViewController* controller = self;
    self.basicChannel = [FlutterBasicMessageChannel messageChannelWithName:@"com.deplay.one.hybrid/basic" binaryMessenger:[registrar messenger] codec:[FlutterStringCodec new]];

}

                         

// 设置methodhandler
// TODO： 这里是一个很长的列表，最好采用callback的方式或者开出api让其他页面可以设置
- (void) setupMethodHandler{
    __weak HybridPlugin *weakSelf = self;
    typeof(weakSelf) __strong strongSelf = weakSelf;
    
    [self.methodChannel setMethodCallHandler:^(FlutterMethodCall * _Nonnull call, FlutterResult  _Nonnull result) {
        if 
        ([@"getPlatformVersion" isEqualToString:call.method]) {
            NSString *platformVersion=[@"iOS " stringByAppendingString:[[UIDevice currentDevice] systemVersion]];
            [weakSelf.basicChannel sendMessage: [weakSelf getJsonMessage:platformVersion method:call.method]];
            result(platformVersion);
        }else if([@"add" isEqualToString:call.method]){
            NSDictionary *args = call.arguments;
            NSLog(@"call arguments form add %@",call.arguments);
            int x = [[args valueForKey:@"x"] intValue];
            int y = [[args valueForKey:@"y"] intValue];
            NSNumber *sum = [NSNumber numberWithLong:(x+y)];
            [weakSelf.basicChannel sendMessage: [weakSelf getJsonMessage:sum.stringValue method:call.method]];
            result(sum);
        }else if([@"attachPage" isEqualToString:call.method]){
            NSLog(@"calling attachPage");
            NSDictionary *args = call.arguments;
            NSString *pageId = [args valueForKey:@"pageId"];
            NSString *script = [args valueForKey:@"script"];
            [strongSelf.pageManager attachPageScriptToJsCore:pageId scriptToAttach:script];
            result(@"success");
        }else if([@"initComplete" isEqualToString:call.method]){
            NSLog(@"calling initComplete");
            NSDictionary *args = call.arguments;
            NSString *pageId = [args valueForKey:@"pageId"];
            // NSLog(@"calling initComplete pageId is %@",pageId);
            [strongSelf.pageManager onInitComplete:pageId];
            result(@"success");
        }
        else if([@"onLoad" isEqualToString:call.method]){
            NSLog(@"calling onLoad");
            NSDictionary *args = call.arguments;
            NSString *pageId = [args valueForKey:@"pageId"];
            NSString *argObject = [args valueForKey:@"args"];
            // NSLog(@"pageId %@",pageId);
            [strongSelf.pageManager callMethodInPage:pageId method:call.method varArg:argObject];
            result(@"success");
        }
        else if([@"onUnload" isEqualToString:call.method]){
            NSLog(@"calling onUnload");
            NSDictionary *args = call.arguments;
            NSString *pageId = [args valueForKey:@"pageId"];
            NSLog(@"pageId %@",pageId);
            // 应该清除定时器
            [strongSelf.pageManager callMethodInPage:pageId method:call.method varArg:NULL];
            [strongSelf.pageManager onRemovePage:pageId];
            result(@"success");
        }
        else if([@"event" isEqualToString:call.method]){
            NSLog(@"calling event");
            NSDictionary *args = call.arguments;
            NSString *pageId = [args valueForKey:@"pageId"];
            NSString *event = [args valueForKey:@"event"];
            NSString *data = [args valueForKey:@"data"];
            // NSLog(@"pageId %@",pageId);
            [strongSelf.pageManager callMethodInPage:pageId method:event varArg:data];
            result(@"success");
        }
        else if([@"removeObserver" isEqualToString:call.method]){
            NSLog(@"calling removeObserver %@",call.arguments);
            NSDictionary *args = call.arguments;
            NSString *pageId = [args valueForKey:@"pageId"];
            NSArray *ids = [args valueForKey:@"ids"];
            // NSLog(@"pageId %@",pageId);
            [strongSelf.pageManager onRemoveObserver:pageId ids:ids];
            result(@"success");
        }
        
        else if([@"handleRepeat" isEqualToString:call.method]){
            NSLog(@"calling handleRepeat");
            NSDictionary *args = call.arguments;
            // NSLog(@"call arguments form attachPage %@",args);
            NSString *pageId = [args valueForKey:@"pageId"];
            NSString *expression=[args valueForKey:@"expression"];
            NSString *componentId=[args valueForKey:@"id"];
            NSString *type=[args valueForKey:@"type"];
            NSString *key=[args valueForKey:@"key"];
            NSNumber *watch=[args valueForKey:@"watch"];
            
            JSValue *obj=[strongSelf.pageManager handleRepeat:pageId componentId:componentId type:type key:key watch:watch expression:expression];
            result(obj.toString);
            // [strongSelf.pageManager getJSPage:pageId];
        }
        else if([@"handleExpression" isEqualToString:call.method]){
            NSLog(@"calling handleExpression");
            NSDictionary *args = call.arguments;
            // NSLog(@"call arguments form attachPage %@",args);
            NSString *pageId = [args valueForKey:@"pageId"];
            NSString *expression=[args valueForKey:@"expression"];
            NSString *componentId=[args valueForKey:@"id"];
            NSString *type=[args valueForKey:@"type"];
            NSString *key=[args valueForKey:@"key"];
            NSNumber *watch=[args valueForKey:@"watch"];
            
            NSString *obj=[strongSelf.pageManager handleExpression:pageId componentId:componentId type:type key:key watch:watch expression:expression];
            result(obj);
            // [strongSelf.pageManager getJSPage:pageId];
        }
        else {
            NSLog(@"method is %@",call.method);
            [weakSelf.basicChannel sendMessage:@"Flutter Method Not Implemented"];
            result(FlutterMethodNotImplemented);
        }
    }];
}


// 通过BasicChannel发送JSON到flutter
// TODO： 这里构造JSON，如果有更好的库，应该考虑替换这里
- (NSString *)getJsonMessage:(NSString*) dataToSend method:(NSString*) methodName
{
    NSMutableDictionary *rt = [[NSMutableDictionary alloc] init];
    [rt setValue:dataToSend forKey:methodName];
    NSError *error = nil;
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:rt
                                                       options:NSJSONWritingPrettyPrinted
                                                         error:&error];
    if (error)
    {
       NSLog(@"error");
    }
    NSString *jsonReturned = [[NSString alloc]initWithData:jsonData encoding:NSUTF8StringEncoding];
    return jsonReturned;
}



@end

